/*
 * Copyright (c) 2011-2012, Yahoo! Inc.  All rights reserved.
 * Copyrights licensed under the New BSD License.
 * See the accompanying LICENSE file for terms.
 */

/*jslint anon:true, sloppy:true, nomen:true*/
/*global YUI*/


/**
 * This object is responsible for running mojits.
 * @class MojitoDispatcher
 * @constructor
 * @param {ServerStore} resourceStore the store to use.
 * @private
 */
YUI.add('mojito-dispatcher', function(Y, NAME) {

    Y.namespace('mojito').Dispatcher = {

        /**
         * Initializes the dispatcher instance.
         * @param {Y.mojito.ResourceStore} resourceStore
         * @return {Y.mojito.Dispatcher}
         */
        init: function(resourceStore) {

            if (!resourceStore) {
                throw new Error(
                    'Mojito cannot instantiate without a resource store.'
                );
            }

            // Cache parameters as instance variables for the dispatch() call to
            // reference.
            this.store = resourceStore;

            this.CACHE = {};

            Y.log('Dispatcher created', 'debug', NAME);

            return this;
        },

        /* See docs for the dispatch function in action-context.common.js */
        dispatch: function(command, adapter) {

            var my = this,
                store = this.store,
                perf = Y.mojito.perf.timeline('mojito',
                    'dispatch:expandInstance',
                    'gather details about mojit', command);

            store.validateContext(command.context);

            store.expandInstance(command.instance, command.context,
                function(err, instance) {
                    var controller,
                        ac;

                    if (err) {
                        throw new Error(err);
                    }

                    perf.done(); // closing 'dispatch:expandInstance' timeline

                    // We replace the given instance with the expanded instance.
                    command.instance = instance;

                    Y.mojito.perf.mark('mojito', 'core_dispatch_start',
                        'dispatching an instance', command);

                    // Ensure there's a createController method we can call
                    // that will always return a viable controller. By
                    // wrapping in a function we allow tests and other code
                    // to provide mocks etc.
                    // TODO: instance.createController should be part of the
                    // expandInstance routine rather than dispatch.
                    instance.createController = instance.createController ||
                        function() {
                            // returning a controller instance instead of
                            // a singleton to avoid leaks.
                            return Y.mojito.util.heir(Y.mojito.controllers[
                                instance.controller
                            ]);
                        };
                    controller = instance.createController();

                    perf = Y.mojito.perf.timeline('mojito', 'ac:ctor',
                        'create ControllerContext', command);

                    // Note that creation of an ActionContext current causes
                    // immediate invocation of the dispatch() call.
                    try {
                        ac = new Y.mojito.ActionContext({
                            command: command,
                            controller: controller,
                            dispatcher: my,         // NOTE passing dispatcher.
                            adapter: adapter,
                            store: store
                        });
                    } catch (e) {
                        Y.log('Error from dispatch on instance \'' +
                            (instance.id || '@' + instance.type) +
                            '\':', 'error', NAME);
                        Y.log(e.message, 'error', NAME);
                        Y.log(e.stack, 'error', NAME);
                        adapter.error(e);
                    }
                    perf.done(); // closing the 'ac:ctor' timeline
                });
        }

    };

}, '0.1.0', {requires: [
    'mojito-action-context',
    'mojito-util'
]});
